@@ -26,20 +26,30 @@
 #include <netinet/in.h>
 #include <time.h>
 
+#define GDM_MAX_PASS 256	/* Define a value for password length. Glibc
+				 * leaves MAX_PASS undefined. */
+
 #define STX 0x2			/* Start of txt */
 #define BEL 0x7			/* Bell, used to interrupt login for
 				 * say timed login or something similar */
 
-#define TYPE_LOCAL 1		/* Local X server */
-#define TYPE_XDMCP 2		/* Remote display */
+#define TYPE_STATIC 1		/* X server defined in gdm.conf */
+#define TYPE_XDMCP 2		/* Remote display/Xserver */
 #define TYPE_FLEXI 3		/* Local Flexi X server */
 #define TYPE_FLEXI_XNEST 4	/* Local Flexi Xnest server */
+#define TYPE_XDMCP_PROXY 5	/* Proxy X server for XDMCP */
 
-#define SERVER_IS_LOCAL(d) ((d)->type == TYPE_LOCAL || \
+#define SERVER_IS_LOCAL(d) ((d)->type == TYPE_STATIC || \
 			    (d)->type == TYPE_FLEXI || \
-			    (d)->type == TYPE_FLEXI_XNEST)
+			    (d)->type == TYPE_FLEXI_XNEST || \
+			    (d)->type == TYPE_XDMCP_PROXY)
 #define SERVER_IS_FLEXI(d) ((d)->type == TYPE_FLEXI || \
-			    (d)->type == TYPE_FLEXI_XNEST)
+			    (d)->type == TYPE_FLEXI_XNEST || \
+			    (d)->type == TYPE_XDMCP_PROXY)
+#define SERVER_IS_PROXY(d) ((d)->type == TYPE_FLEXI_XNEST || \
+			    (d)->type == TYPE_XDMCP_PROXY)
+#define SERVER_IS_XDMCP(d) ((d)->type == TYPE_XDMCP || \
+			    (d)->type == TYPE_XDMCP_PROXY)
 
 /* These are the servstat values, also used as server
  * process exit codes */
@@ -71,7 +81,8 @@ enum {
 	DISPLAY_ALIVE /* Yay! we're alive (non-xdmcp) */,
 	XDMCP_PENDING /* Pending XDMCP display */,
 	XDMCP_MANAGED /* Managed XDMCP display */,
-	DISPLAY_DEAD /* Left for dead */
+	DISPLAY_DEAD /* Left for dead */,
+	DISPLAY_CONFIG /* in process of being configured */
 };
 
 /* Opcodes for the highly sophisticated protocol used for
@@ -115,25 +126,30 @@ enum {
 #define GDM_INTERRUPT_SELECT_USER 'U'
 #define GDM_INTERRUPT_LOGIN_SOUND 'L'
 #define GDM_INTERRUPT_THEME       'H'
+#define GDM_INTERRUPT_CANCEL      'X'
+
+/* List delimiter for config file lists */
+#define GDM_DELIMITER_MODULES ":"
+#define GDM_DELIMITER_THEMES "/:"
 
 /* The dreaded miscellaneous category */
 #define FIELD_SIZE 256
 #define PIPE_SIZE 4096
 
 /* Configuration constants */
-#define GDM_KEY_CHOOSER "daemon/Chooser=" EXPANDED_BINDIR "/gdmchooser"
+#define GDM_KEY_CHOOSER "daemon/Chooser=" EXPANDED_LIBEXECDIR "/gdmchooser"
 /* This defaults to true for backward compatibility,
  * it will not actually do automatic login since the AutomaticLogin defaults
  * to nothing */
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
 #define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=false"
-#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
-#define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
+#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_LIBEXECDIR "/gdmlogin"
+#define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_LIBEXECDIR "/gdmlogin"
 #define GDM_KEY_ADD_GTK_MODULES "daemon/AddGtkModules=false"
 #define GDM_KEY_GTK_MODULES_LIST "daemon/GtkModulesList="
 #define GDM_KEY_GROUP "daemon/Group=gdm"
-#define GDM_KEY_HALT "daemon/DefaultPath=" HALT_COMMAND
+#define GDM_KEY_HALT "daemon/HaltCommand=" HALT_COMMAND
 #define GDM_KEY_INITDIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
 #define GDM_KEY_KILLIC "daemon/KillInitClients=true"
 #define GDM_KEY_LOGDIR "daemon/LogDir=" EXPANDED_LOGDIR
@@ -156,6 +172,7 @@ enum {
 #define GDM_KEY_UAUTHFB "daemon/UserAuthFBDir=/tmp"
 #define GDM_KEY_UAUTHFILE "daemon/UserAuthFile=.Xauthority"
 #define GDM_KEY_USER "daemon/User=gdm"
+#define GDM_KEY_CONSOLE_NOTIFY "daemon/ConsoleNotify=true"
 
 #define GDM_KEY_DOUBLELOGINWARNING "daemon/DoubleLoginWarning=true"
 
@@ -172,6 +189,7 @@ enum {
 
 #define GDM_KEY_STANDARD_XSERVER "daemon/StandardXServer=" X_SERVER
 #define GDM_KEY_FLEXIBLE_XSERVERS "daemon/FlexibleXServers=5"
+#define GDM_KEY_DYNAMIC_XSERVERS "daemon/DynamicXServers=false"
 #define GDM_KEY_XNEST "daemon/Xnest=" X_SERVER_PATH "/Xnest -name Xnest"
 /* Keys for automatic VT allocation rather then letting it up to the
  * X server */
@@ -181,6 +199,7 @@ enum {
 #define GDM_KEY_CONSOLE_CANNOT_HANDLE "daemon/ConsoleCannotHandle=am,ar,az,bn,el,fa,gu,hi,ja,ko,ml,mr,pa,ta,zh"
 
 /* Per server definitions */
+#define GDM_KEY_SERVER_PREFIX "server-"
 #define GDM_KEY_SERVER_NAME "name=Standard server"
 #define GDM_KEY_SERVER_COMMAND "command=" X_SERVER
 /* runnable as flexi server */
@@ -215,6 +234,10 @@ enum {
 #define GDM_KEY_PINGINTERVAL "xdmcp/PingIntervalSeconds=15"
 #define GDM_KEY_WILLING "xdmcp/Willing=" EXPANDED_SYSCONFDIR "/gdm/Xwilling"
 
+#define GDM_KEY_XDMCP_PROXY "xdmcp/EnableProxy=false"
+#define GDM_KEY_XDMCP_PROXY_XSERVER "xdmcp/ProxyXServer="
+#define GDM_KEY_XDMCP_PROXY_RECONNECT "xdmcp/ProxyReconnect="
+
 #define GDM_KEY_GTK_THEME "gui/GtkTheme=Default"
 #define GDM_KEY_GTKRC "gui/GtkRC="
 #define GDM_KEY_ICONWIDTH "gui/MaxIconWidth=128"
@@ -224,23 +247,30 @@ enum {
 #define GDM_KEY_GTK_THEMES_TO_ALLOW "gui/GtkThemesToAllow=all"
 
 #define GDM_KEY_BROWSER "greeter/Browser=false"
+#define GDM_KEY_INCLUDE "greeter/Include="
 #define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm,rpm,nfsnobody,pcap"
+#define GDM_KEY_INCLUDEALL "greeter/IncludeAll=false"
 #define GDM_KEY_MINIMALUID "greeter/MinimalUID=100"
 #define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "/nobody.png"
-#define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
+#define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/pixmaps/faces/"
 #define GDM_KEY_LOCFILE "greeter/LocaleFile=" EXPANDED_LOCALEDIR "/locale.alias"
 #define GDM_KEY_LOGO "greeter/Logo=" EXPANDED_PIXMAPDIR "/gdm-foot-logo.png"
 #define GDM_KEY_QUIVER "greeter/Quiver=true"
 #define GDM_KEY_SYSMENU "greeter/SystemMenu=true"
-#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmsetup --disable-sound --disable-crash-dialog"
+#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_SBINDIR "/gdmsetup --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
 #define GDM_KEY_CHOOSER_BUTTON "greeter/ChooserButton=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
-#define GDM_KEY_WELCOME "greeter/Welcome=Welcome"
-#define GDM_KEY_REMOTEWELCOME "greeter/RemoteWelcome=Welcome to %n"
+#define GDM_KEY_DEFAULT_WELCOME "greeter/DefaultWelcome=true"
+#define GDM_KEY_DEFAULT_REMOTEWELCOME "greeter/DefaultRemoteWelcome=true"
+#define GDM_KEY_WELCOME "greeter/Welcome="
+#define GDM_KEY_REMOTEWELCOME "greeter/RemoteWelcome="
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
 #define GDM_KEY_RUNBACKGROUNDPROGALWAYS "greeter/RunBackgroundProgramAlways=false"
+#define GDM_KEY_BACKGROUNDPROGINITIALDELAY "greeter/BackgroundProgramInitialDelay=30"
+#define GDM_KEY_RESTARTBACKGROUNDPROG "greeter/RestartBackgroundProgram=true"
+#define GDM_KEY_BACKGROUNDPROGRESTARTDELAY "greeter/BackgroundProgramRestartDelay=30"
 #define GDM_KEY_BACKGROUNDIMAGE "greeter/BackgroundImage="
 #define GDM_KEY_BACKGROUNDCOLOR "greeter/BackgroundColor=#76848F"
 #define GDM_KEY_BACKGROUNDTYPE "greeter/BackgroundType=2"
@@ -254,13 +284,19 @@ enum {
 #define GDM_KEY_ENTRY_CIRCLES "greeter/UseCirclesInEntry=false"
 #define GDM_KEY_ENTRY_INVISIBLE "greeter/UseInvisibleInEntry=false"
 #define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
+#define GDM_KEY_GRAPHICAL_THEMES "greeter/GraphicalThemes=circles"
+#define GDM_KEY_GRAPHICAL_THEME_RAND "greeter/GraphicalThemeRand=false"
 #define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
 
 #define GDM_KEY_INFO_MSG_FILE "greeter/InfoMsgFile="
 #define GDM_KEY_INFO_MSG_FONT "greeter/InfoMsgFont="
 
-#define GDM_KEY_SOUND_ON_LOGIN "greeter/SoundOnLogin=true"
-#define GDM_KEY_SOUND_ON_LOGIN_FILE "greeter/SoundOnLoginFile="
+#define GDM_KEY_SOUND_ON_LOGIN_READY "greeter/SoundOnLogin=true"
+#define GDM_KEY_SOUND_ON_LOGIN_SUCCESS "greeter/SoundOnLoginSuccess=false"
+#define GDM_KEY_SOUND_ON_LOGIN_FAILURE "greeter/SoundOnLoginFailure=false"
+#define GDM_KEY_SOUND_ON_LOGIN_READY_FILE "greeter/SoundOnLoginFile="
+#define GDM_KEY_SOUND_ON_LOGIN_SUCCESS_FILE "greeter/SoundOnLoginSuccessFile="
+#define GDM_KEY_SOUND_ON_LOGIN_FAILURE_FILE "greeter/SoundOnLoginFailureFile="
 #define GDM_KEY_SOUND_PROGRAM "daemon/SoundProgram=/usr/bin/play"
 
 #define GDM_KEY_SCAN "chooser/ScanTime=4"
@@ -292,6 +328,11 @@ enum {
 
 #define GDM_STANDARD "Standard"
 
+#define GDM_DEFAULT_WELCOME_MSG "Welcome"
+#define GDM_DEFAULT_REMOTEWELCOME_MSG "Welcome to %n"
+
+#define GDM_RESPONSE_CANCEL "GDM_RESPONSE_CANCEL"
+
 #ifndef TYPEDEF_GDM_CONNECTION
 #define TYPEDEF_GDM_CONNECTION
 typedef struct _GdmConnection GdmConnection;
@@ -350,6 +391,7 @@ struct _GdmDisplay {
 
     guint8 dispstat;
     guint16 dispnum;
+	gboolean removeconf;
     guint8 servstat;
     time_t starttime;
     time_t managetime;
@@ -369,7 +411,8 @@ struct _GdmDisplay {
 
     gboolean busy_display;
 
-    gboolean console;
+    gboolean attached;  /* Display is physically attached to the machine. */
+                        /* TYPE_XDMCP would have this FALSE, eg. */
 
     time_t last_start_time;
     time_t last_loop_start_time;
@@ -395,12 +438,15 @@ struct _GdmDisplay {
     int lrh_offsety; /* lower right hand corner y offset */
 
     /* Flexi stuff */
-    char *xnest_disp;
-    char *xnest_auth_file;
-    char *xnest_temp_auth_file;
+    char *parent_disp;
+    Display *parent_dsp;
+    char *parent_auth_file;
+    char *parent_temp_auth_file;
     uid_t server_uid;
     GdmConnection *socket_conn;
 
+    int xdmcp_dispnum;
+
     /* Notification connection */
     int master_notify_fd;  /* write part of the connection */
     int slave_notify_fd; /* read part of the connection */
@@ -517,11 +563,13 @@ void		gdm_final_cleanup	(void);
 #define GDM_SOP_LOGGED_IN    "LOGGED_IN" /* <slave pid> <logged_in as int> */
 #define GDM_SOP_LOGIN        "LOGIN" /* <slave pid> <username> */
 #define GDM_SOP_COOKIE       "COOKIE" /* <slave pid> <cookie> */
+#define GDM_SOP_AUTHFILE     "AUTHFILE" /* <slave pid> <authfile> */
 #define GDM_SOP_QUERYLOGIN   "QUERYLOGIN" /* <slave pid> <username> */
 /* if user already logged in somewhere, the ack response will be
-   <display>,<vt>,<display>,<vt>,... */
+   <display>,<migratable>,<display>,<migratable>,... */
+#define GDM_SOP_MIGRATE      "MIGRATE" /* <slave pid> <display> */
 #define GDM_SOP_DISP_NUM     "DISP_NUM" /* <slave pid> <display as int> */
-/* For linux only currently */
+/* For Linux only currently */
 #define GDM_SOP_VT_NUM       "VT_NUM" /* <slave pid> <vt as int> */
 #define GDM_SOP_FLEXI_ERR    "FLEXI_ERR" /* <slave pid> <error num> */
 	/* 3 = X failed */
@@ -576,7 +624,9 @@ void		gdm_final_cleanup	(void);
 #define GDM_NOTIFY_TIMED_LOGIN "TimedLogin" /* <login> */
 #define GDM_NOTIFY_TIMED_LOGIN_DELAY "TimedLoginDelay" /* <seconds> */
 #define GDM_NOTIFY_DISALLOWTCP "DisallowTCP" /* <true/false as int> */
-#define GDM_NOTIFY_SOUND_ON_LOGIN_FILE "SoundOnLoginFile" /* <sound file> */
+#define GDM_NOTIFY_SOUND_ON_LOGIN_READY_FILE "SoundOnLoginFile" /* <sound file> */
+#define GDM_NOTIFY_SOUND_ON_LOGIN_SUCCESS_FILE "SoundOnLoginSuccessFile" /* <sound file> */
+#define GDM_NOTIFY_SOUND_ON_LOGIN_FAILURE_FILE "SoundOnLoginFailureFile" /* <sound file> */
 #define GDM_NOTIFY_ADD_GTK_MODULES "AddGtkModules" /* <true/false as int> */
 #define GDM_NOTIFY_GTK_MODULES_LIST "GtkModulesList" /* <modules list> */
 
@@ -691,20 +741,28 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
-#define GDM_SUP_CONSOLE_SERVERS  "CONSOLE_SERVERS" /* None */
-/* CONSOLE_SERVERS: List all console servers, useful for linux mostly
- *  Doesn't list xdmcp and xnest non-console servers
+#define GDM_SUP_ATTACHED_SERVERS "ATTACHED_SERVERS" /* None */
+#define GDM_SUP_CONSOLE_SERVERS  "CONSOLE_SERVERS"  /* None */
+/* ATTACHED_SERVERS: List all attached servers, useful for Linux mostly
+ *   Doesn't list xdmcp and xnest non-attached servers
+ * CONSOLE_SERVERS supported, but deprecated due to terminology
  * Supported since: 2.2.4.0
- * Arguments:  None
+ * Note: This command used to be named CONSOLE_SERVERS, which is still recognized
+ *       for backwards compatibility.  The optional pattern argument is supported
+ *       as of version 2.8.0.0.
+ * Arguments:  <pattern> (optional)
+ *   With no argument, all dynamic displays are returned. The optional
+ *   <pattern> is a string that may contain glob characters '*', '?', and
+ *   '[]'. Only displays that match the pattern will be returned.
  * Answers:
  *   OK <server>;<server>;...
  *
  *   <server> is <display>,<logged in user>,<vt or xnest display>
  *
  *   <logged in user> can be empty in case no one logged in yet,
- *   and <vt> can be -1 if it's not known or not supported (on non-linux
- *   for example).  If the display is an xnest display and is a console one
- *   (that is, it is an xnest inside another console display) it is listed
+ *   and <vt> can be -1 if it's not known or not supported (on non-Linux
+ *   for example).  If the display is an xnest display and is an attached one
+ *   (that is, it is an xnest inside another attached display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
  *
  *   ERROR <err number> <english error description>
@@ -713,7 +771,7 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_ALL_SERVERS  "ALL_SERVERS" /* None */
-/* ALL_SERVERS: List all servers, including console, remote, xnest.  This
+/* ALL_SERVERS: List all servers, including attached, remote, xnest.  This
  * Can for example be useful to figure out if the server you are on is managed
  * by the gdm daemon, by seeing if it is in the list.  It is also somewhat
  * like the 'w' command but for graphical sessions.
@@ -731,6 +789,21 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_GET_CONFIG "GET_CONFIG" /* <key> */
+/* GET_CONFIG:  Get configuration value for key.  Useful so
+ * that other programs can request configuration information
+ * from GDM.  Any key defined as GDM_KEY_* in gdm.h is 
+ * supported.
+ * Supported since: 2.6.0.9
+ * Arguments:   <key>
+ * Answers:
+ *   OK <value>
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      50 = Unsupported key
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
 #define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
 /* UPDATE_CONFIG: Tell the daemon to update config of some key.  Any user
  *             can really request that values are re-read but the daemon
@@ -800,7 +873,7 @@ void		gdm_final_cleanup	(void);
  *      over SET_SAFE_LOGOUT_ACTION.
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
- *      100 = Not authenticanted
+ *      100 = Not authenticated
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -820,7 +893,7 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      7 = Unknown logout action, or not available
- *      100 = Not authenticanted
+ *      100 = Not authenticated
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -844,7 +917,7 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      7 = Unknown logout action, or not available
- *      100 = Not authenticanted
+ *      100 = Not authenticated
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -868,7 +941,7 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      8 = Virtual terminals not supported
- *      100 = Not authenticanted
+ *      100 = Not authenticated
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -886,7 +959,7 @@ void		gdm_final_cleanup	(void);
  *      0 = Not implemented
  *      8 = Virtual terminals not supported
  *      9 = Invalid virtual terminal number
- *      100 = Not authenticanted
+ *      100 = Not authenticated
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -895,12 +968,61 @@ void		gdm_final_cleanup	(void);
  * Supported since: 2.2.4.0
  */
 
+#define GDM_SUP_ADD_DYNAMIC_DISPLAY	"ADD_DYNAMIC_DISPLAY" 
+/*
+ * ADD_X_SERVER: Add a dynamic display configuration.
+ *   Configures a dynamic X server to run on the specified display
+ *   leaving it in DISPLAY_CONFIG state.
+ * Supported since: 2.8.0.0
+ * Arguments:  <display to run on>=<server>
+ *   Where <server> is either a configuration named in gdm.conf or
+ *   a literal command name.
+ * Answers:
+ *  OK
+ *  ERROR
+ *     0 = Not implemented
+ *     999 = Unknown error
+ */
+#define GDM_SUP_REMOVE_DYNAMIC_DISPLAY	"REMOVE_DYNAMIC_DISPLAY" 
+/*
+ * REMOVE_X_SERVER: Remove a dynamic display
+ *   Removes a dynamic display, killing the server and purging
+ *   the display configuration
+ * Supported since: 2.8.0.0
+ * Arguments:  <display to remove>
+ * Answers:
+ *  OK
+ *  ERROR
+ *     0 = Not implemented
+ *     999 = Unknown error
+ */
+#define GDM_SUP_RELEASE_DYNAMIC_DISPLAYS	"RELEASE_DYNAMIC_DISPLAYS"
+/*
+ * RELEASE_SERVERS: Release dynamic displays currently in PAUSED state
+ * Supported since: 2.8.0.0
+ * Arguments: None
+ * Answers:
+ *  OK
+ *  ERROR
+ *     0 = Not implemented
+ *     999 = Unknown error
+ */
+
 /* User flags for the SUP protocol */
 enum {
-	GDM_SUP_FLAG_AUTHENTICATED = 0x1 /* authenticated as a local user,
+	GDM_SUP_FLAG_AUTHENTICATED = 0x1, /* authenticated as a local user,
 					  * from a local display we started */
+	GDM_SUP_FLAG_AUTH_GLOBAL = 0x2 /* authenticated with global cookie */
 };
 
+/* Macros to check authentication level */
+#define GDM_CONN_AUTHENTICATED(conn) \
+	((gdm_connection_get_user_flags (conn) & GDM_SUP_FLAG_AUTHENTICATED) || \
+	 (gdm_connection_get_user_flags (conn) & GDM_SUP_FLAG_AUTH_GLOBAL))
+
+#define GDM_CONN_AUTH_GLOBAL(conn) \
+	 (gdm_connection_get_user_flags (conn) & GDM_SUP_FLAG_AUTH_GLOBAL)
+
 
 #define NEVER_FAILS_seteuid(uid) \
 	{ int r = 0; \
