@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,7 @@ extern gboolean GdmDebug;
 extern GSList *displays;
 extern int gdm_xdmcpfd;
 extern gboolean GdmXdmcp;
+extern gchar *GdmConsoleCannotHandle;
 
 extern char **environ;
 
@@ -66,22 +67,120 @@ extern pid_t gdm_main_pid;
 extern gboolean preserve_ld_vars;
 extern int gdm_in_signal;
 
-extern gboolean no_console;
+extern gboolean GdmConsoleNotify;
 
 extern char *gdm_charset;
 
 #ifdef ENABLE_IPV6
+
+#ifdef sun
+static gboolean 
+have_ipv6_solaris (void)
+{
+          int            s, i;
+          int            ret;
+          struct lifnum  ln; 
+          struct lifconf ifc;
+          struct lifreq *ifr;   
+          char          *ifreqs;
+          
+          /* First, try the <AB>classic<BB> way
+           */
+          s = socket (AF_INET6, SOCK_DGRAM, 0);
+          if (s < 0) return FALSE;
+          close(s);
+
+          s = socket (AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+
+          /* Ok, the system is able to create IPv6 sockets, so
+           * lets check if IPv6 is configured in the machine
+           */
+          ln.lifn_family=AF_UNSPEC;
+          ln.lifn_flags=ln.lifn_count=0; 
+
+          ret = ioctl(s, SIOCGLIFNUM, &ln);
+          if (ret == -1) {
+                        perror("ioctl SIOCGLIFNUM");
+                        return FALSE;
+          }
+
+          /* Alloc the memory and get the configuration
+           */
+          ifc.lifc_flags  = 0; 
+          ifc.lifc_family = AF_UNSPEC;
+          ifc.lifc_len    = ln.lifn_count * sizeof(struct lifreq);
+
+          ifreqs = (char *) malloc (ifc.lifc_len);
+          ifc.lifc_buf = ifreqs;
+
+          if (ioctl (s, SIOCGLIFCONF, &ifc) < 0) {
+                        perror("ioctl SIOCGLIFCONF");
+                        return FALSE;
+          }
+
+          /* Check each interface
+           */
+          ifr  = ifc.lifc_req;
+          ret  = FALSE;
+
+          for (i = ifc.lifc_len/sizeof (struct lifreq); --i >= 0; ifr++) {
+                struct sockaddr_in *sin;
+
+                        /* Check the address
+                         */
+                        if (ioctl (s, SIOCGLIFFLAGS, ifr) < 0) {
+                                   // perror("ioctl SIOCGLIFADDR");
+                                   continue;
+                        }
+
+                        sin = (struct sockaddr_in *)&ifr->lifr_addr;
+
+                        if (sin->sin_family == AF_INET6) {
+                                   ret = TRUE;
+                                   break;
+                        }
+
+                        /* Check the interface flags
+                         */
+                        if (ioctl (s, SIOCGLIFFLAGS, (char *) ifr) < 0) {
+                                   // perror("ioctl SIOCGLIFFLAGS");
+                                   continue;
+                        }
+
+                        if (ifr->lifr_flags & IFF_IPV6) {      
+                                   ret = TRUE;
+                                   break;
+                        }
+          }
+          
+          /* Clean up
+           */
+          free (ifreqs);
+          close(s);
+
+          return ret;
+}
+#endif
+
 static gboolean have_ipv6 (void)
 {
 	int s;
+        static gboolean has_ipv6 = -1;
 
-	s = socket (AF_INET6, SOCK_STREAM, 0);
-	if (s != -1) {
-		VE_IGNORE_EINTR (close (s));
-		return TRUE;
+#ifdef sun
+        has_ipv6 = have_ipv6_solaris();
+#else
+        if (has_ipv6 != -1) return has_ipv6;
+
+        s = socket (AF_INET6, SOCK_STREAM, 0);  
+        if (s < 0) {
+                  has_ipv6 = FALSE;
+                  return FALSE;
 	}
 
-	return FALSE;
+       VE_IGNORE_EINTR (close (s));            
+#endif
+       return has_ipv6;
 }
 #endif
 
@@ -486,19 +585,33 @@ gdm_text_message_dialog (const char *msg)
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
 
-	if (no_console)
+    if ( ! GdmConsoleNotify)
 		return FALSE;
 
 	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
-	msg_quoted = g_shell_quote (msg);
+	if (msg[0] == '-') {
+		char *tmp = g_strconcat (" ", msg, NULL);
+		msg_quoted = g_shell_quote (tmp);
+		g_free (tmp);
+	} else {
+		msg_quoted = g_shell_quote (msg);
+	}
 	
 	dialog = g_find_program_in_path ("dialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("whiptail");
 	if (dialog != NULL) {
 		char *argv[6];
+
+		if ( ! gdm_ok_console_language ()) {
+			ve_unsetenv ("LANG");
+			ve_unsetenv ("LC_ALL");
+			ve_unsetenv ("LC_MESSAGES");
+			ve_setenv ("LANG", "C", TRUE);
+			ve_setenv ("UNSAFE_TO_TRANSLATE", "yes", TRUE);
+		}
 		
 		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
@@ -550,7 +663,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
 
-	if (no_console)
+    if ( ! GdmConsoleNotify)
 		return FALSE;
 	
 	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
@@ -559,7 +672,13 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 	if (ret != NULL)
 		*ret = FALSE;
 
-	msg_quoted = g_shell_quote (msg);
+	if (msg[0] == '-') {
+		char *tmp = g_strconcat (" ", msg, NULL);
+		msg_quoted = g_shell_quote (tmp);
+		g_free (tmp);
+	} else {
+		msg_quoted = g_shell_quote (msg);
+	}
 	
 	dialog = g_find_program_in_path ("dialog");
 	if (dialog == NULL)
@@ -568,6 +687,14 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		char *argv[6];
 		int retint;
 
+		if ( ! gdm_ok_console_language ()) {
+			ve_unsetenv ("LANG");
+			ve_unsetenv ("LC_ALL");
+			ve_unsetenv ("LC_MESSAGES");
+			ve_setenv ("LANG", "C", TRUE);
+			ve_setenv ("UNSAFE_TO_TRANSLATE", "yes", TRUE);
+		}
+
 		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
@@ -2295,4 +2422,54 @@ gdm_get_last_info (const char *username)
 	return info;
 }
 
+gboolean
+gdm_ok_console_language (void)
+{
+	int i;
+	char **v;
+	static gboolean cached = FALSE;
+	static gboolean is_ok;
+	const char *loc;
+
+	if (cached)
+		return is_ok;
+
+	/* so far we should be paranoid, we're not set yet */
+	if (GdmConsoleCannotHandle == NULL)
+		return FALSE;
+
+	cached = TRUE;
+
+	loc = setlocale (LC_MESSAGES, NULL);
+	if (loc == NULL) {
+		is_ok = TRUE;
+		return TRUE;
+	}
+
+	is_ok = TRUE;
+
+	v = g_strsplit (GdmConsoleCannotHandle, ",", -1);
+	for (i = 0; v != NULL && v[i] != NULL; i++) {
+		if ( ! ve_string_empty (v[i]) &&
+		    strncmp (v[i], loc, strlen (v[i])) == 0) {
+			is_ok = FALSE;
+			break;
+		}
+	}
+	if (v != NULL)
+		g_strfreev (v);
+
+	return is_ok;
+}
+
+const char *
+gdm_console_translate (const char *str)
+{
+	if (gdm_ok_console_language ())
+		return _(str);
+	else
+		return str;
+}
+
+
 /* EOF */
