@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -96,12 +96,12 @@ gdm_server_whack_lockfile (GdmDisplay *disp)
 
 	    /* if lock file exists and it is our process, whack it! */
 	    g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", disp->dispnum);
-	    IGNORE_EINTR (unlink (buf));
+	    VE_IGNORE_EINTR (unlink (buf));
 
 	    /* whack the unix socket as well */
 	    g_snprintf (buf, sizeof (buf),
 			"/tmp/.X11-unix/X%d", disp->dispnum);
-	    IGNORE_EINTR (unlink (buf));
+	    VE_IGNORE_EINTR (unlink (buf));
 }
 
 
@@ -109,12 +109,14 @@ gdm_server_whack_lockfile (GdmDisplay *disp)
 void
 gdm_server_wipe_cookies (GdmDisplay *disp)
 {
-	if ( ! ve_string_empty (disp->authfile))
-		IGNORE_EINTR (unlink (disp->authfile));
+	if ( ! ve_string_empty (disp->authfile)) {
+		VE_IGNORE_EINTR (unlink (disp->authfile));
+	}
 	g_free (disp->authfile);
 	disp->authfile = NULL;
-	if ( ! ve_string_empty (disp->authfile_gdm))
-		IGNORE_EINTR (unlink (disp->authfile_gdm));
+	if ( ! ve_string_empty (disp->authfile_gdm)) {
+		VE_IGNORE_EINTR (unlink (disp->authfile_gdm));
+	}
 	g_free (disp->authfile_gdm);
 	disp->authfile_gdm = NULL;
 }
@@ -134,6 +136,32 @@ jumpback_xioerror_handler (Display *disp)
 	Longjmp (reinitjmp, 1);
 }
 
+#ifdef HAVE_FBCONSOLE
+#define FBCONSOLE "/usr/openwin/bin/fbconsole"
+
+static void
+gdm_exec_fbconsole(GdmDisplay *disp)
+{
+        pid_t pid;
+        char *argv[6];
+
+        argv[0] = FBCONSOLE;
+        argv[1] = "-d";
+        argv[2] = disp->name;
+        argv[3] = NULL;
+
+        pid = fork ();
+        if (pid == 0) {
+                gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */)
+;
+                VE_IGNORE_EINTR (execv (argv[0], argv));
+        }
+        if (pid == -1) {
+                gdm_error (_("Can not start fallback console"));
+        }
+}
+#endif
+
 /**
  * gdm_server_reinit:
  * @disp: Pointer to a GdmDisplay structure
@@ -215,10 +243,17 @@ gdm_server_reinit (GdmDisplay *disp)
 	/* Wait for the SIGUSR1 */
 	do_server_wait (d);
 
-	if (d->servstat == SERVER_RUNNING)
+	if (d->servstat == SERVER_RUNNING) {
+#ifdef HAVE_FBCONSOLE
+		gdm_exec_fbconsole(d);
+#endif
 		return TRUE;
-	else
+        } else {
+		/* if something really REALLY screwed up, then whack the
+		   lockfiles for safety */
+		gdm_server_whack_lockfile (d);
 		return FALSE;
+	}
 }
 
 /**
@@ -232,6 +267,7 @@ void
 gdm_server_stop (GdmDisplay *disp)
 {
     static gboolean waiting_for_server = FALSE;
+    int old_servstat;
 
     if (disp == NULL)
 	return;
@@ -240,16 +276,26 @@ gdm_server_stop (GdmDisplay *disp)
     if (disp->dsp != NULL) {
 	    /* on XDMCP servers first kill everything in sight */
 	    if (disp->type == TYPE_XDMCP)
-		    gdm_server_whack_clients (d);
+		    gdm_server_whack_clients (disp->dsp);
 	    XCloseDisplay (disp->dsp);
 	    disp->dsp = NULL;
     }
 
+    /* Kill our parent connection if one existed */
+    if (disp->parent_dsp != NULL) {
+	    /* on XDMCP servers first kill everything in sight */
+	    if (disp->type == TYPE_XDMCP_PROXY)
+		    gdm_server_whack_clients (disp->parent_dsp);
+	    XCloseDisplay (disp->parent_dsp);
+	    disp->parent_dsp = NULL;
+    }
+
     if (disp->servpid <= 0)
 	    return;
 
     gdm_debug ("gdm_server_stop: Server for %s going down!", disp->name);
 
+    old_servstat = disp->servstat;
     disp->servstat = SERVER_DEAD;
 
     if (disp->servpid > 0) {
@@ -271,7 +317,7 @@ gdm_server_stop (GdmDisplay *disp)
 		    if (disp->servpid > 1 &&
 			kill (disp->servpid, SIGTERM) == 0) {
 			    waiting_for_server = TRUE;
-			    ve_waitpid_no_signal (disp->servpid, 0, 0);
+			    ve_waitpid_no_signal (disp->servpid, NULL, 0);
 			    waiting_for_server = FALSE;
 		    }
 	    }
@@ -279,9 +325,15 @@ gdm_server_stop (GdmDisplay *disp)
 
 	    gdm_sigchld_block_pop ();
 
-	    gdm_server_whack_lockfile (disp);
+	    if (old_servstat == SERVER_RUNNING)
+		    gdm_server_whack_lockfile (disp);
 
 	    gdm_debug ("gdm_server_stop: Server pid %d dead", (int)servpid);
+
+	    /* just in case we restart again wait at least
+	       one sec to avoid races */
+	    if (d->sleep_before_run < 1)
+		    d->sleep_before_run = 1;
     }
 
     gdm_server_wipe_cookies (disp);
@@ -295,17 +347,18 @@ busy_ask_user (GdmDisplay *disp)
     /* if we have "open" we can talk to the user */
     if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
-		    (_("There already appears to be an X server "
-		       "running on display %s.  Should I try another "
-		       "display number?  If you answer no, I will "
-		       "attempt to start the server on %s again.%s"),
+		    (C_(N_("There already appears to be an X server "
+			   "running on display %s.  Should another "
+			   "display number by tried?  Answering no will "
+			   "cause GDM to attempt starting the server "
+			   "on %s again.%s")),
 		     disp->name,
 		     disp->name,
 #ifdef __linux__
-		     _("  (You can change consoles by pressing Ctrl-Alt "
-		       "plus a function key, such as Ctrl-Alt-F7 to go "
-		       "to console 7.  X servers usually run on consoles "
-		       "7 and higher.)")
+		     C_(N_("  (You can change consoles by pressing Ctrl-Alt "
+			   "plus a function key, such as Ctrl-Alt-F7 to go "
+			   "to console 7.  X servers usually run on consoles "
+			   "7 and higher.)"))
 #else /* ! __linux__ */
 		     /* no info for non linux users */
 		     ""
@@ -325,61 +378,67 @@ busy_ask_user (GdmDisplay *disp)
 
 /* Checks only output, no XFree86 v4 logfile */
 static gboolean
-display_xnest_no_connect (GdmDisplay *disp)
+display_parent_no_connect (GdmDisplay *disp)
 {
-	char *logname = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+	char *logname = gdm_make_filename (GdmLogDir, d->name, ".log");
 	FILE *fp;
 	char buf[256];
+	char *getsret;
 
-	fp = fopen (logname, "r");
+	VE_IGNORE_EINTR (fp = fopen (logname, "r"));
 	g_free (logname);
 
 	if (fp == NULL)
 		return FALSE;
 
-	while (fgets (buf, sizeof (buf), fp) != NULL) {
+	for (;;) {
+		VE_IGNORE_EINTR (getsret = fgets (buf, sizeof (buf), fp));
+		if (getsret == NULL) {
+			VE_IGNORE_EINTR (fclose (fp));
+			return FALSE;
+		}
 		/* Note: this is probably XFree86 specific, and perhaps even
 		 * version 3 specific (I don't have xfree v4 to test this),
 		 * of course additions are welcome to make this more robust */
 		if (strstr (buf, "Unable to open display \"") == buf) {
 			gdm_error (_("Display '%s' cannot be opened by Xnest"),
-				   ve_sure_string (disp->xnest_disp));
-			fclose (fp);
+				   ve_sure_string (disp->parent_disp));
+			VE_IGNORE_EINTR (fclose (fp));
 			return TRUE;
 		}
 	}
-
-	fclose (fp);
-	return FALSE;
 }
 
 static gboolean
 display_busy (GdmDisplay *disp)
 {
-	char *logname = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+	char *logname = gdm_make_filename (GdmLogDir, d->name, ".log");
 	FILE *fp;
 	char buf[256];
+	char *getsret;
 
-	fp = fopen (logname, "r");
+	VE_IGNORE_EINTR (fp = fopen (logname, "r"));
 	g_free (logname);
 
 	if (fp == NULL)
 		return FALSE;
 
-	while (fgets (buf, sizeof (buf), fp) != NULL) {
+	for (;;) {
+		VE_IGNORE_EINTR (getsret = fgets (buf, sizeof (buf), fp));
+		if (getsret == NULL) {
+			VE_IGNORE_EINTR (fclose (fp));
+			return FALSE;
+		}
 		/* Note: this is probably XFree86 specific */
 		if (strstr (buf, "Server is already active for display")
 		    == buf) {
 			gdm_error (_("Display %s is busy. There is another "
 				     "X server running already."),
 				   disp->name);
-			fclose (fp);
+			VE_IGNORE_EINTR (fclose (fp));
 			return TRUE;
 		}
 	}
-
-	fclose (fp);
-	return FALSE;
 }
 
 /* if we find 'Log file: "foo"' switch fp to foo and
@@ -395,10 +454,10 @@ open_another_logfile (char buf[256], FILE **fp)
 		if (p == NULL)
 			return FALSE;
 		*p = '\0';
-		ffp = fopen (fname, "r");
+		VE_IGNORE_EINTR (ffp = fopen (fname, "r"));
 		if (ffp == NULL)
 			return FALSE;
-		fclose (*fp);
+		VE_IGNORE_EINTR (fclose (*fp));
 		*fp = ffp;
 		return TRUE;
 	}
@@ -408,21 +467,28 @@ open_another_logfile (char buf[256], FILE **fp)
 static int
 display_vt (GdmDisplay *disp)
 {
-	char *logname = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+	char *logname = gdm_make_filename (GdmLogDir, d->name, ".log");
 	FILE *fp;
 	char buf[256];
 	gboolean switched = FALSE;
+	char *getsret;
 
-	fp = fopen (logname, "r");
+	VE_IGNORE_EINTR (fp = fopen (logname, "r"));
 	g_free (logname);
 
 	if (fp == NULL)
 		return FALSE;
 
-	while (fgets (buf, sizeof (buf), fp) != NULL) {
+	for (;;) {
 		int vt;
 		char *p;
 
+		VE_IGNORE_EINTR (getsret = fgets (buf, sizeof (buf), fp));
+		if (getsret == NULL) {
+			VE_IGNORE_EINTR (fclose (fp));
+			return -1;
+		}
+
 		if ( ! switched &&
 		     /* this is XFree v4 specific */
 		    open_another_logfile (buf, &fp)) {
@@ -434,12 +500,10 @@ display_vt (GdmDisplay *disp)
 		p = strstr (buf, "using VT number ");
 		if (p != NULL &&
 		    sscanf (p, "using VT number %d", &vt) == 1) {
-			fclose (fp);
+			VE_IGNORE_EINTR (fclose (fp));
 			return vt;
 		}
 	}
-	fclose (fp);
-	return -1;
 }
 
 static struct sigaction old_svr_wait_chld;
@@ -466,8 +530,8 @@ setup_server_wait (GdmDisplay *d)
     if (sigaction (SIGUSR1, &usr1, NULL) < 0) {
 	    gdm_error (_("%s: Error setting up %s signal handler: %s"),
 		       "gdm_server_start", "USR1", strerror (errno));
-	    IGNORE_EINTR (close (server_signal_pipe[0]));
-	    IGNORE_EINTR (close (server_signal_pipe[1]));
+	    VE_IGNORE_EINTR (close (server_signal_pipe[0]));
+	    VE_IGNORE_EINTR (close (server_signal_pipe[1]));
 	    return FALSE;
     }
 
@@ -480,8 +544,8 @@ setup_server_wait (GdmDisplay *d)
 	    gdm_error (_("%s: Error setting up %s signal handler: %s"),
 		       "gdm_server_start", "CHLD", strerror (errno));
 	    gdm_signal_ignore (SIGUSR1);
-	    IGNORE_EINTR (close (server_signal_pipe[0]));
-	    IGNORE_EINTR (close (server_signal_pipe[1]));
+	    VE_IGNORE_EINTR (close (server_signal_pipe[0]));
+	    VE_IGNORE_EINTR (close (server_signal_pipe[1]));
 	    return FALSE;
     }
 
@@ -499,7 +563,7 @@ do_server_wait (GdmDisplay *d)
 {
     /* Wait for X server to send ready signal */
     if (d->servstat == SERVER_PENDING) {
-	    if (d->server_uid != 0 && ! d->handled) {
+	    if (d->server_uid != 0 && ! d->handled && ! d->chosen_hostname) {
 		    /* FIXME: If not handled, we just don't know, so
 		     * just wait a few seconds and hope things just work,
 		     * fortunately there is no such case yet and probably
@@ -556,7 +620,7 @@ do_server_wait (GdmDisplay *d)
 			    if (select (server_signal_pipe[0]+1, &rfds, NULL, NULL, &tv) > 0) {
 				    char buf[4];
 				    /* read the Yay! */
-				    IGNORE_EINTR (read (server_signal_pipe[0], buf, 4));
+				    VE_IGNORE_EINTR (read (server_signal_pipe[0], buf, 4));
 			    }
 			    if ( ! server_signal_notified &&
 				t + SERVER_WAIT_ALARM < time (NULL)) {
@@ -579,12 +643,71 @@ do_server_wait (GdmDisplay *d)
     sigaction (SIGCHLD, &old_svr_wait_chld, NULL);
     sigprocmask (SIG_SETMASK, &old_svr_wait_mask, NULL);
 
-    IGNORE_EINTR (close (server_signal_pipe[0]));
-    IGNORE_EINTR (close (server_signal_pipe[1]));
+    VE_IGNORE_EINTR (close (server_signal_pipe[0]));
+    VE_IGNORE_EINTR (close (server_signal_pipe[1]));
 
     if (d->servpid <= 1) {
 	    d->servstat = SERVER_ABORT;
     }
+
+    if (d->servstat != SERVER_RUNNING) {
+	    /* bad things are happening */
+	    if (d->servpid > 0) {
+		    pid_t pid;
+
+		    d->dsp = NULL;
+
+		    gdm_sigchld_block_push ();
+		    pid = d->servpid;
+		    d->servpid = 0;
+		    if (pid > 1 &&
+			kill (pid, SIGTERM) == 0)
+			    ve_waitpid_no_signal (pid, NULL, 0);
+		    gdm_sigchld_block_pop ();
+	    }
+
+	    /* We will rebake cookies anyway, so wipe these */
+	    gdm_server_wipe_cookies (d);
+    }
+}
+
+/* We keep a connection (parent_dsp) open with the parent X server
+ * before running a proxy on it to prevent the X server resetting
+ * as we open and close other connections.
+ * Note that XDMCP servers, by default, reset when the seed X
+ * connection closes whereas usually the X server only quits when
+ * all X connections have closed.
+ */
+static gboolean
+connect_to_parent (GdmDisplay *d)
+{
+	int maxtries;
+	int openretries;
+
+	gdm_debug ("gdm_server_start: Connecting to parent display \'%s\'",
+		   d->parent_disp);
+
+	d->parent_dsp = NULL;
+
+	maxtries = SERVER_IS_XDMCP (d) ? 10 : 2;
+
+	openretries = 0;
+	while (openretries < maxtries &&
+	       d->parent_dsp == NULL) {
+		d->parent_dsp = XOpenDisplay (d->parent_disp);
+
+		if G_UNLIKELY (d->parent_dsp == NULL) {
+			gdm_debug ("gdm_server_start: Sleeping %d on a retry", 1+openretries*2);
+			gdm_sleep_no_signal (1+openretries*2);
+			openretries++;
+		}
+	}
+
+	if (d->parent_dsp == NULL)
+		gdm_error (_("%s: failed to connect to parent display '\%s\'"),
+			   "gdm_server_start", d->parent_disp);
+
+	return d->parent_dsp != NULL;
 }
 
 /**
@@ -595,8 +718,11 @@ do_server_wait (GdmDisplay *d)
  */
 
 gboolean
-gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
-		  int min_flexi_disp, int flexi_retries)
+gdm_server_start (GdmDisplay *disp,
+		  gboolean try_again_if_busy /* only affects non-flexi servers */,
+		  gboolean treat_as_flexi,
+		  int min_flexi_disp,
+		  int flexi_retries)
 {
     int flexi_disp = 20;
     char *vtarg = NULL;
@@ -611,7 +737,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     gdm_server_stop (d);
 
     /* First clear the VT number */
-    if (d->type == TYPE_LOCAL ||
+    if (d->type == TYPE_STATIC ||
 	d->type == TYPE_FLEXI) {
 	    d->vt = -1;
 	    gdm_slave_send_num (GDM_SOP_VT_NUM, -1);
@@ -630,6 +756,9 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 	    gdm_slave_send_num (GDM_SOP_DISP_NUM, flexi_disp);
     }
 
+    if (d->type == TYPE_XDMCP_PROXY &&
+	! connect_to_parent (d))
+	    return FALSE;
 
     gdm_debug ("gdm_server_start: %s", d->name);
 
@@ -637,6 +766,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     if ( ! gdm_auth_secure_display (d)) 
 	    return FALSE;
     gdm_slave_send_string (GDM_SOP_COOKIE, d->cookie);
+    gdm_slave_send_string (GDM_SOP_AUTHFILE, d->authfile);
     ve_setenv ("DISPLAY", d->name, TRUE);
 
     if ( ! setup_server_wait (d))
@@ -644,7 +774,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     d->servstat = SERVER_DEAD;
 
-    if (d->type == TYPE_LOCAL ||
+    if (d->type == TYPE_STATIC ||
 	d->type == TYPE_FLEXI) {
 	    vtarg = gdm_get_empty_vt_argument (&vtfd, &vt);
     }
@@ -653,13 +783,13 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     gdm_server_spawn (d, vtarg);
 
     /* we can now use d->handled since that's set up above */
-
     do_server_wait (d);
 
     /* If we were holding a vt open for the server, close it now as it has
      * already taken the bait. */
-    if (vtfd > 0)
-	    IGNORE_EINTR (close (vtfd));
+    if (vtfd > 0) {
+	    VE_IGNORE_EINTR (close (vtfd));
+    }
 
     switch (d->servstat) {
 
@@ -676,7 +806,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
 	    if (SERVER_IS_FLEXI (d))
 		    gdm_slave_send_num (GDM_SOP_FLEXI_OK, 0 /* bogus */);
-	    if (d->type == TYPE_LOCAL ||
+	    if (d->type == TYPE_STATIC ||
 		d->type == TYPE_FLEXI) {
 		    if (vt >= 0)
 			    d->vt = vt;
@@ -687,35 +817,19 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    gdm_slave_send_num (GDM_SOP_VT_NUM, d->vt);
 	    }
 
+#ifdef HAVE_FBCONSOLE
+            gdm_exec_fbconsole(d);
+#endif
+
 	    return TRUE;
     default:
 	    break;
     }
 
-    /* bad things are happening */
-    if (d->servpid > 0) {
-	    pid_t pid;
-
-	    d->dsp = NULL;
-
-	    gdm_sigchld_block_push ();
-	    pid = d->servpid;
-	    d->servpid = 0;
-	    if (pid > 1 &&
-		kill (pid, SIGTERM) == 0)
-		    ve_waitpid_no_signal (pid, NULL, 0);
-	    gdm_sigchld_block_pop ();
-	    
-	    gdm_server_whack_lockfile (disp);
-    }
-
-    /* We will rebake cookies anyway, so wipe these */
-    gdm_server_wipe_cookies (disp);
-
-    if (disp->type == TYPE_FLEXI_XNEST &&
-	display_xnest_no_connect (disp)) {
+    if (SERVER_IS_PROXY (disp) &&
+	display_parent_no_connect (disp)) {
 	    gdm_slave_send_num (GDM_SOP_FLEXI_ERR,
-				5 /* Xnest can't connect */);
+				5 /* proxy can't connect */);
 	    _exit (DISPLAY_REMANAGE);
     }
 
@@ -740,10 +854,22 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    /* eki eki */
 			    _exit (DISPLAY_REMANAGE);
 		    }
-		    return gdm_server_start (d, treat_as_flexi,
+		    return gdm_server_start (d, FALSE /*try_again_if_busy */,
+					     treat_as_flexi,
 					     flexi_disp + 1,
 					     flexi_retries - 1);
 	    } else {
+		    if (try_again_if_busy) {
+			    gdm_debug ("%s: Display %s busy.  Trying once again "
+				       "(after 2 sec delay)",
+				       "gdm_server_start", d->name);
+			    gdm_sleep_no_signal (2);
+			    return gdm_server_start (d,
+						     FALSE /* try_again_if_busy */,
+						     treat_as_flexi,
+						     flexi_disp,
+						     flexi_retries);
+		    }
 		    if (busy_ask_user (disp)) {
 			    gdm_error (_("%s: Display %s busy.  Trying "
 					 "another display number."),
@@ -751,6 +877,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 				       d->name);
 			    d->busy_display = TRUE;
 			    return gdm_server_start (d,
+						     FALSE /*try_again_if_busy */,
 						     TRUE /* treat as flexi */,
 						     high_display_num + 1,
 						     flexi_retries - 1);
@@ -771,7 +898,7 @@ void
 gdm_server_checklog (GdmDisplay *disp)
 {
 	if (d->vt < 0 &&
-	    (d->type == TYPE_LOCAL ||
+	    (d->type == TYPE_STATIC ||
 	     d->type == TYPE_FLEXI)) {
 		d->vt = display_vt (d);
 		if (d->vt >= 0)
@@ -788,30 +915,30 @@ safer_rename (const char *a, const char *b)
 	errno = 0;
 	if (link (a, b) < 0) {
 		if (errno == EEXIST) {
-			IGNORE_EINTR (unlink (a));
+			VE_IGNORE_EINTR (unlink (a));
 			return;
 		} 
-		IGNORE_EINTR (unlink (b));
+		VE_IGNORE_EINTR (unlink (b));
 		/* likely this system doesn't support hard links */
 		rename (a, b);
-		IGNORE_EINTR (unlink (a));
+		VE_IGNORE_EINTR (unlink (a));
 		return;
 	}
-	IGNORE_EINTR (unlink (a));
+	VE_IGNORE_EINTR (unlink (a));
 }
 
 static void
 rotate_logs (const char *dname)
 {
 	/* I'm too lazy to write a loop damnit */
-	char *fname4 = g_strconcat (GdmLogDir, "/", dname, ".log.4", NULL);
-	char *fname3 = g_strconcat (GdmLogDir, "/", dname, ".log.3", NULL);
-	char *fname2 = g_strconcat (GdmLogDir, "/", dname, ".log.2", NULL);
-	char *fname1 = g_strconcat (GdmLogDir, "/", dname, ".log.1", NULL);
-	char *fname = g_strconcat (GdmLogDir, "/", dname, ".log", NULL);
+	char *fname4 = gdm_make_filename (GdmLogDir, dname, ".log.4");
+	char *fname3 = gdm_make_filename (GdmLogDir, dname, ".log.3");
+	char *fname2 = gdm_make_filename (GdmLogDir, dname, ".log.2");
+	char *fname1 = gdm_make_filename (GdmLogDir, dname, ".log.1");
+	char *fname = gdm_make_filename (GdmLogDir, dname, ".log");
 
 	/* Rotate the logs (keep 4 last) */
-	IGNORE_EINTR (unlink (fname4));
+	VE_IGNORE_EINTR (unlink (fname4));
 	safer_rename (fname3, fname4);
 	safer_rename (fname2, fname3);
 	safer_rename (fname1, fname2);
@@ -824,6 +951,20 @@ rotate_logs (const char *dname)
 	g_free (fname);
 }
 
+GdmXServer *
+gdm_server_resolve (GdmDisplay *disp)
+{
+	char *bin;
+	GdmXServer *svr = NULL;
+
+	bin = ve_first_word (disp->command);
+	if (bin != NULL && bin[0] != '/') {
+		svr = gdm_find_x_server (bin);
+	}
+	g_free (bin);
+	return svr;
+}
+
 
 char **
 gdm_server_resolve_command_line (GdmDisplay *disp,
@@ -844,7 +985,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 	} else if (bin[0] != '/') {
 		GdmXServer *svr = gdm_find_x_server (bin);
 		if (svr == NULL) {
-			gdm_error (_("Server name '%s' not found, "
+			gdm_error (_("Server name '%s' not found; "
 				     "using standard server"), bin);
 			argv = ve_split (GdmStandardXServer);
 		} else {
@@ -918,6 +1059,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 	if (resolve_flags && GdmDisallowTCP && ! query_in_arglist) {
 		argv[len++] = g_strdup ("-nolisten");
 		argv[len++] = g_strdup ("tcp");
+		d->tcp_disallowed = TRUE;
 	}
 
 	if (vtarg != NULL &&
@@ -1011,19 +1153,19 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	rotate_logs (d->name);
 
         /* Log all output from spawned programs to a file */
-	logfile = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
-	IGNORE_EINTR (unlink (logfile));
-	logfd = open (logfile, O_CREAT|O_TRUNC|O_WRONLY|O_EXCL, 0644);
+	logfile = gdm_make_filename (GdmLogDir, d->name, ".log");
+	VE_IGNORE_EINTR (unlink (logfile));
+	VE_IGNORE_EINTR (logfd = open (logfile, O_CREAT|O_TRUNC|O_WRONLY|O_EXCL, 0644));
 
 	if (logfd != -1) {
-		IGNORE_EINTR (dup2 (logfd, 1));
-		IGNORE_EINTR (dup2 (logfd, 2));
+		VE_IGNORE_EINTR (dup2 (logfd, 1));
+		VE_IGNORE_EINTR (dup2 (logfd, 2));
+		close (logfd);
         } else {
 		gdm_error (_("%s: Could not open logfile for display %s!"),
 			   "gdm_server_spawn", d->name);
 	}
 
-	
 	/* The X server expects USR1/TTIN/TTOU to be SIG_IGN */
 	ign_signal.sa_handler = SIG_IGN;
 	ign_signal.sa_flags = SA_RESTART;
@@ -1056,27 +1198,36 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&mask);
 	sigprocmask (SIG_SETMASK, &mask, NULL);
 
-	if (d->type == TYPE_FLEXI_XNEST) {
-		char *font_path = NULL;
-		ve_setenv ("DISPLAY", d->xnest_disp, TRUE);
-		if (d->xnest_auth_file != NULL)
-			ve_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
+	if (SERVER_IS_PROXY (d)) {
+		int argc = ve_vector_len (argv);
+
+		ve_unsetenv ("DISPLAY");
+		if (d->parent_auth_file != NULL)
+			ve_setenv ("XAUTHORITY", d->parent_auth_file, TRUE);
 		else
 			ve_unsetenv ("XAUTHORITY");
 
-		/* Add -fp with the current font path, but only if not
-		 * already among the arguments */
-		if (strstr (command, "-fp") == NULL)
-			font_path = get_font_path (d->xnest_disp);
-		if (font_path != NULL) {
-			int argc = ve_vector_len (argv);
-			argv = g_renew (char *, argv, argc + 3);
-			argv[argc++] = "-fp";
-			argv[argc++] = font_path;
-			argv[argc++] = NULL;
-			command = g_strconcat (command, " -fp ",
-					       font_path, NULL);
+		if (d->type == TYPE_FLEXI_XNEST) {
+			char *font_path = NULL;
+			/* Add -fp with the current font path, but only if not
+			 * already among the arguments */
+			if (strstr (command, "-fp") == NULL)
+				font_path = get_font_path (d->parent_disp);
+			if (font_path != NULL) {
+				argv = g_renew (char *, argv, argc + 2);
+				argv[argc++] = "-fp";
+				argv[argc++] = font_path;
+				command = g_strconcat (command, " -fp ",
+						       font_path, NULL);
+			}
 		}
+
+		argv = g_renew (char *, argv, argc + 3);
+		argv[argc++] = "-display";
+		argv[argc++] = d->parent_disp;
+		argv[argc++] = NULL;
+		command = g_strconcat (command, " -display ",
+				       d->parent_disp, NULL);
 	}
 
 	if (argv[0] == NULL) {
@@ -1137,8 +1288,14 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		setgroups (1, groups);
 	}
 
-	IGNORE_EINTR (execv (argv[0], argv));
-	
+	VE_IGNORE_EINTR (execv (argv[0], argv));
+
+	gdm_fdprintf (2, "GDM: Xserver not found: %s\n"
+		      "Error: Command could not be executed!\n"
+		      "Please install the X server or edit %s to point "
+		      "to the right place.",
+		      command, GDM_CONFIG_FILE);
+
 	gdm_error (_("%s: Xserver not found: %s"), 
 		   "gdm_server_spawn", command);
 	
@@ -1178,11 +1335,9 @@ gdm_server_usr1_handler (gint sig)
     d->servstat = SERVER_RUNNING; /* Server ready to accept connections */
     d->starttime = time (NULL);
 
-    gdm_debug ("gdm_server_usr1_handler: Got SIGUSR1, server running");
-
     server_signal_notified = TRUE;
     /* this will quit the select */
-    IGNORE_EINTR (write (server_signal_pipe[1], "Yay!", 4));
+    VE_IGNORE_EINTR (write (server_signal_pipe[1], "Yay!", 4));
 
     gdm_in_signal--;
 }
@@ -1200,13 +1355,11 @@ gdm_server_child_handler (int signal)
 {
 	gdm_in_signal++;
 
-	gdm_debug ("gdm_server_child_handler: Got SIGCHLD");
-
 	/* go to the main child handler */
 	gdm_slave_child_handler (signal);
 
 	/* this will quit the select */
-	IGNORE_EINTR (write (server_signal_pipe[1], "Yay!", 4));
+	VE_IGNORE_EINTR (write (server_signal_pipe[1], "Yay!", 4));
 
 	gdm_in_signal--;
 }
@@ -1232,6 +1385,8 @@ gdm_server_alloc (gint id, const gchar *command)
 
     d = g_new0 (GdmDisplay, 1);
 
+    d->logout_action = GDM_LOGOUT_ACTION_NONE;
+
     d->authfile = NULL;
     d->authfile_gdm = NULL;
     d->auths = NULL;
@@ -1243,14 +1398,14 @@ gdm_server_alloc (gint id, const gchar *command)
     d->name = g_strdup_printf (":%d", id);  
     d->hostname = g_strdup (hostname);
     /* Not really used for not XDMCP */
-    memset (&(d->addr), 0, sizeof (struct in_addr));
+    memset (&(d->addr), 0, sizeof (d->addr));
     d->dispnum = id;
     d->servpid = 0;
     d->servstat = SERVER_DEAD;
     d->sesspid = 0;
     d->slavepid = 0;
-    d->type = TYPE_LOCAL;
-    d->console = TRUE;
+    d->type = TYPE_STATIC;
+    d->attached = TRUE;
     d->sessionid = 0;
     d->acctime = 0;
     d->dsp = NULL;
@@ -1258,6 +1413,7 @@ gdm_server_alloc (gint id, const gchar *command)
     d->screeny = 0;
 
     d->handled = TRUE;
+    d->tcp_disallowed = FALSE;
 
     d->vt = -1;
 
@@ -1266,7 +1422,6 @@ gdm_server_alloc (gint id, const gchar *command)
     d->last_loop_start_time = 0;
     d->last_start_time = 0;
     d->retry_count = 0;
-    d->disabled = FALSE;
     d->sleep_before_run = 0;
     d->login = NULL;
 
@@ -1274,48 +1429,56 @@ gdm_server_alloc (gint id, const gchar *command)
 
     d->slave_notify_fd = -1;
     d->master_notify_fd = -1;
+
+    d->xsession_errors_bytes = 0;
+    d->xsession_errors_fd = -1;
+    d->session_output_fd = -1;
+
+    d->chooser_output_fd = -1;
+    d->chooser_last_line = NULL;
+
+    d->theme_name = NULL;
     
     return d;
 }
 
 void
-gdm_server_whack_clients (GdmDisplay *disp)
+gdm_server_whack_clients (Display *dsp)
 {
 	int i, screen_count;
 	int (* old_xerror_handler) (Display *, XErrorEvent *);
 
-	if (disp == NULL ||
-	    disp->dsp == NULL)
+	if (dsp == NULL)
 		return;
 
 	old_xerror_handler = XSetErrorHandler (ignore_xerror_handler);
 
-	XGrabServer (disp->dsp);
+	XGrabServer (dsp);
 
-	screen_count = ScreenCount (disp->dsp);
+	screen_count = ScreenCount (dsp);
 
 	for (i = 0; i < screen_count; i++) {
 		Window root_ret, parent_ret;
 		Window *childs = NULL;
 		unsigned int childs_count = 0;
-		Window root = RootWindow (disp->dsp, i);
+		Window root = RootWindow (dsp, i);
 
-		while (XQueryTree (disp->dsp, root, &root_ret, &parent_ret,
+		while (XQueryTree (dsp, root, &root_ret, &parent_ret,
 				   &childs, &childs_count) &&
 		       childs_count > 0) {
 			int ii;
 
 			for (ii = 0; ii < childs_count; ii++) {
-				XKillClient (disp->dsp, childs[ii]);
+				XKillClient (dsp, childs[ii]);
 			}
 
 			XFree (childs);
 		}
 	}
 
-	XUngrabServer (disp->dsp);
+	XUngrabServer (dsp);
 
-	XSync (disp->dsp, False);
+	XSync (dsp, False);
 	XSetErrorHandler (old_xerror_handler);
 }
 
