@@ -28,19 +28,21 @@ void		gdm_server_wipe_cookies	(GdmDisplay *disp);
 void		gdm_server_whack_lockfile (GdmDisplay *disp);
 
 gboolean	gdm_server_start	(GdmDisplay *d,
+					 gboolean try_again_if_busy,
 					 gboolean treat_as_flexi,
 					 int min_flexi_disp,
 					 int flexi_retries);
 void		gdm_server_stop		(GdmDisplay *d);
-void		gdm_server_reinit	(GdmDisplay *d);
+gboolean	gdm_server_reinit	(GdmDisplay *d);
 GdmDisplay *	gdm_server_alloc	(gint id,
 					 const gchar *command);
-void		gdm_server_whack_clients (GdmDisplay *disp);
+void		gdm_server_whack_clients (Display *dsp);
 void		gdm_server_checklog	(GdmDisplay *disp);
 
 char **		gdm_server_resolve_command_line (GdmDisplay *disp,
 						 gboolean resolve_flags,
 						 const char *vtarg);
+GdmXServer *	gdm_server_resolve	(GdmDisplay *disp);
 
 
 
