@@ -27,7 +27,7 @@ void     gdm_slave_start       (GdmDisplay *d);
 void     gdm_slave_greeter_ctl_no_ret (char cmd, const char *str);
 char    *gdm_slave_greeter_ctl (char cmd, const char *str);
 gboolean gdm_slave_greeter_check_interruption (void);
-gboolean gdm_slave_should_complain (void);
+gboolean gdm_slave_action_pending (void);
 
 void	 gdm_slave_send		(const char *str, gboolean wait_for_ack);
 void	 gdm_slave_send_num	(const char *opcode, long num);
@@ -38,6 +38,8 @@ void     gdm_slave_whack_temp_auth_file (void);
 
 gboolean gdm_slave_check_user_wants_to_log_in (const char *user);
 
+/* This is the slave child handler so that we can chain to it from elsewhere */
+void	 gdm_slave_child_handler (int sig);
 
 #endif /* GDM_SLAVE_H */
 
