@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -58,7 +58,7 @@ extern gboolean GdmDisplayLastLogin;
 extern uid_t GdmUserId;
 extern gid_t GdmGroupId;
 
-extern gboolean no_console;
+extern gboolean GdmConsoleNotify;
 
 /* Evil, but this way these things are passed to the child session */
 static pam_handle_t *pamh = NULL;
@@ -149,7 +149,7 @@ audit_success_login(int pw_change, struct passwd *pwent)
  *
  *	Entry	did_setcred == FALSE, process audit context is not established.
  *			       TRUE, process audit context established.
- *		d = display structure, d->console non 0 if local,
+ *		d = display structure, d->attached non 0 if local,
  *			d->hostname used if remote.
  *		pw_change == PW_FALSE, if no password change requested.
  *			     PW_TRUE, if successful password change audit
@@ -186,9 +186,9 @@ audit_fail_login(GdmDisplay *d, int pw_change, struct passwd *pwent,
 			    "adt_start_session(ADT_login, ADT_FAILURE): %m");
 			return;
 		}
-		if (d->console) {
+		if (d->attached) {
 			/* login from the local host */
-			if (adt_load_hostname(NULL, &tid) != 0) {
+			if (adt_load_ttyname("/dev/console", &tid) != 0) {
 
 				syslog(LOG_AUTH | LOG_ALERT,
 				    "adt_loadhostname(localhost): %m");
@@ -674,7 +674,7 @@ create_pamh (GdmDisplay *d,
 
 	/* Inform PAM of the user's tty */
 #ifdef	sun
-	if (d->console == 0)
+	if (d->attached == 0)
 		(void) pam_set_item (pamh, PAM_TTY, "/dev/console");
 	else 
 #endif	/* sun */
@@ -684,7 +684,7 @@ create_pamh (GdmDisplay *d,
 		return FALSE;
 	}
 
-	if ( ! d->console) {
+	if ( ! d->attached) {
 		/* Only set RHOST if host is remote */
 		/* From the host of the display */
 		if ((*pamerr = pam_set_item (pamh, PAM_RHOST,
@@ -946,9 +946,9 @@ authenticate_again:
        	! gdm_setup_gids (login, pwent->pw_gid)) {
 	    gdm_error (_("Cannot set user group for %s"), login);
 	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-					  _("\nCannot set your user group, "
-					    "you will not be able to log in, "
-					    "please contact your system administrator."));
+					  _("\nCannot set your user group; "
+					    "you will not be able to log in. "
+					    "Please contact your system administrator."));
 #ifdef  HAVE_ADT
 		/*
 		 * map group setup error as a pam_setcred() failure
@@ -993,7 +993,7 @@ authenticate_again:
     tmp_PAM_USER = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->console)
+	if (d->attached)
 		(void) di_devperm_login("/dev/console", pwent->pw_uid,
 		    pwent->pw_gid, NULL);
 #endif	/* HAVE_LOGINDEVPERM */
@@ -1216,7 +1216,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	gdm_error (_("User %s not permitted to gain access at this time"), login);
 	gdm_error_box (cur_gdm_disp,
 		       GTK_MESSAGE_ERROR,
-		       _("\nThe system administrator has disabled your access to the system temporary."));
+		       _("\nThe system administrator has disabled your access to the system temporarily."));
 	goto setup_pamerr;
     default :
 	if (gdm_slave_action_pending ())
@@ -1230,9 +1230,9 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	    gdm_error (_("Cannot set user group for %s"), login);
 	    gdm_error_box (cur_gdm_disp,
 			   GTK_MESSAGE_ERROR,
-			   _("\nCannot set your user group, "
-			     "you will not be able to log in, "
-			     "please contact your system administrator."));
+			   _("\nCannot set your user group; "
+			     "you will not be able to log in. "
+			     "Please contact your system administrator."));
 #ifdef  HAVE_ADT
 		/*
 		 * map group setup error as a pam_setcred() failure
@@ -1280,7 +1280,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     extra_standalone_message = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->console)
+	if (d->attached)
 		(void) di_devperm_login("/dev/console", pwent->pw_uid,
 		    pwent->pw_gid, NULL);
 #endif	/* HAVE_LOGINDEVPERM */
@@ -1382,7 +1382,7 @@ gdm_verify_cleanup (GdmDisplay *d)
 		pam_end (tmp_pamh, pamerr);
 
 #ifdef  HAVE_LOGINDEVPERM
-        if (old_opened_session && old_did_setcred && d->console)
+        if (old_opened_session && old_did_setcred && d->attached)
         {
             (void) di_devperm_logout("/dev/console");
             /* give it back to gdm user */
@@ -1428,7 +1428,7 @@ gdm_verify_check (void)
 		closelog ();
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
-		if ( ! no_console)
+        if (GdmConsoleNotify)
 			gdm_text_message_dialog
 				(C_(N_("Can't find PAM configuration for gdm.")));
 		gdm_fail ("gdm_verify_check: %s",
