@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,10 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#if defined(CAN_CLEAR_ADMCHG) && defined(HAVE_USERSEC_H)
+#  include <usersec.h>
+#endif /* CAN_CLEAR_ADMCHG && HAVE_USERSEC_H */
+
 #ifdef HAVE_CRYPT
 #  include <crypt.h>
 #endif /* HAVE_CRYPT */
@@ -139,7 +143,7 @@ authenticate_again:
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
 
 	    if (GdmDisplayLastLogin) {
-		    char *info = gdm_get_last_info (s);
+		    char *info = gdm_get_last_info (login);
 		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, info);
 		    g_free (info);
 	    }
@@ -291,9 +295,9 @@ authenticate_again:
     if ( ! gdm_setup_gids (login, pwent->pw_gid)) {
 	    gdm_error (_("Cannot set user group for %s"), login);
 	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-					  _("\nCannot set your user group, "
-					    "you will not be able to log in, "
-					    "please contact your system administrator."));
+					  _("\nCannot set your user group; "
+					    "you will not be able to log in. "
+					    "Please contact your system administrator."));
 	    g_free (login);
 	    return NULL;
     }
@@ -315,9 +319,9 @@ authenticate_again:
 		    if (ret != 1) {
 			    if (ret != 0) {
 				    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-								  _("\nCannot change your password, "
-								    "you will not be able to log in, "
-								    "please try again later or contact "
+								  _("\nCannot change your password; "
+								    "you will not be able to log in. "
+								    "Please try again later or contact "
 								    "your system administrator."));
 			    } else if ((reEnter != 0) && (message)) {
 				    response = gdm_slave_greeter_ctl (GDM_NOECHO, message);
@@ -339,6 +343,40 @@ authenticate_again:
 		    return NULL;
 	    }
 
+#if defined(CAN_CLEAR_ADMCHG)
+	    /* The password is changed by root, clear the ADM_CHG
+	       flag in the passwd file */
+	    ret = setpwdb (S_READ | S_WRITE);
+	    if (!ret) {
+		    upwd = getuserpw (login);
+		    if (upwd == NULL) {
+			    ret = -1;
+		    }
+		    else {
+			    upwd->upw_flags &= ~PW_ADMCHG;
+			    ret = putuserpw (upwd);
+			    if (!ret) {
+				    ret = endpwdb ();
+			    }
+		    }
+	    }
+
+	    if (ret) {
+		    gdm_error_box (d, GTK_MESSAGE_WARNING,
+				   _("Your password has been changed but "
+				     "you may have to change it again. "
+				     "Please try again later or contact "
+				     "your system administrator."));
+	    }
+
+#else /* !CAN_CLEAR_ADMCHG */
+	    gdm_error_box (d, GTK_MESSAGE_WARNING,
+			   _("Your password has been changed but you "
+			     "may have to change it again. Please try again "
+			     "later or contact your system administrator."));
+
+#endif /* CAN_CLEAR_ADMCHG */
+
 	    break;
 
     case 2 :
@@ -353,7 +391,7 @@ authenticate_again:
     case -1 :
 	    gdm_error (_("Internal error on passwdexpired"));
 	    gdm_error_box (d, GTK_MESSAGE_ERROR,
-			   _("An internal error occured, you will not be able to log in.\n"
+			   _("An internal error occurred. You will not be able to log in.\n"
 			     "Please try again later or contact your system administrator."));
 	    g_free (info_msg);
 	    return NULL;
@@ -396,9 +434,9 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 		gdm_error (_("Cannot set user group for %s"), login);
 		gdm_error_box (d,
 			       GTK_MESSAGE_ERROR,
-			       _("\nCannot set your user group, "
-				 "you will not be able to log in, "
-				 "please contact your system administrator."));
+			       _("\nCannot set your user group; "
+				 "you will not be able to log in. "
+				 "Please contact your system administrator."));
 		return FALSE;
 	}
 
