@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K, Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -53,6 +53,7 @@
 #include "gdm.h"
 #include "misc.h"
 #include "gdmwm.h"
+#include "gdmcommon.h"
 
 static gboolean RUNNING_UNDER_GDM = FALSE;
 
@@ -166,6 +167,19 @@ static gboolean GdmMulticast;
 static gchar *GdmMulticastAddr;
 #endif
 
+/* HACK: for gdmcommon, else it complains */
+/* */  gchar *GdmInfoMsgFile;
+/* */  gchar *GdmInfoMsgFont;
+/* */  gchar *GdmSoundProgram;
+/* */  gboolean GdmSoundOnLoginReady;
+/* */  gchar *GdmSoundOnLoginReadyFile;
+/* */  void
+/* */  gdm_kill_thingies (void)
+/* */  {
+/* */	  ; /* nothing */
+/* */  }
+
+
 enum {
 	GDM_BACKGROUND_NONE = 0,
 	GDM_BACKGROUND_IMAGE = 1,
@@ -1375,8 +1389,8 @@ gdm_chooser_add_host (void)
 					 GTK_BUTTONS_OK,
 					 FALSE,
 					 _("Cannot find host"),
-					 _("I cannot find the host \"%s\", "
-					   "perhaps you have mistyped it."),
+					 _("Cannot find the host \"%s\". "
+					   "Perhaps you have mistyped it."),
 					 name);
 
 					if (RUNNING_UNDER_GDM)
@@ -1808,6 +1822,11 @@ gdm_chooser_gui_init (void)
     gtk_window_resize (GTK_WINDOW (chooser), 
 		       width, height);
 
+
+    /* cursor blinking is evil on remote displays, don't do it forever */
+    gdm_setup_blinking ();
+    gdm_setup_blinking_entry (add_entry);
+
     if (RUNNING_UNDER_GDM) {
 	    gtk_widget_show_now (chooser);
 	    gdm_wm_center_window (GTK_WINDOW (chooser));
@@ -2073,7 +2092,7 @@ main (int argc, char *argv[])
 					_("The chooser version (%s) does not match the daemon "
 					  "version (%s).  "
 					  "You have probably just upgraded gdm.  "
-					  "Please restart the gdm daemon or reboot the computer."),
+					  "Please restart the gdm daemon or the computer."),
 					VERSION, gdm_version);
 
 	    gtk_widget_show_all (dialog);
@@ -2105,15 +2124,13 @@ main (int argc, char *argv[])
     gdm_chooser_xdmcp_init (hosts);
     poptFreeContext (ctx);
 
-    if (RUNNING_UNDER_GDM) {
-	    guint sid = g_signal_lookup ("event",
-					 GTK_TYPE_WIDGET);
-	    g_signal_add_emission_hook (sid,
-					0 /* detail */,
-					gdm_event,
-					NULL /* data */,
-					NULL /* destroy_notify */);
-    }
+    guint sid = g_signal_lookup ("event",
+				 GTK_TYPE_WIDGET);
+    g_signal_add_emission_hook (sid,
+				0 /* detail */,
+				gdm_event,
+				NULL /* data */,
+				NULL /* destroy_notify */);
 
     gtk_widget_queue_resize (chooser);
     gtk_widget_show_now (chooser);
