@@ -1,6 +1,25 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
+#include <librsvg/rsvg.h>
 #include "gdmwm.h"
 #include "greeter_geometry.h"
 #include "greeter_canvas_item.h"
@@ -467,6 +486,7 @@ greeter_item_size_request (GreeterItemInfo *item,
     case GREETER_ITEM_SIZE_BOX:
       set_width = box_requisition.width;
       break;
+    case GREETER_ITEM_SIZE_SCALE:
     case GREETER_ITEM_SIZE_UNSET:
       break;
     }
@@ -482,6 +502,7 @@ greeter_item_size_request (GreeterItemInfo *item,
     case GREETER_ITEM_SIZE_BOX:
       set_height = box_requisition.height;
       break;
+    case GREETER_ITEM_SIZE_SCALE:
     case GREETER_ITEM_SIZE_UNSET:
       break;
     }
@@ -528,6 +549,24 @@ greeter_item_size_request (GreeterItemInfo *item,
       req->width = gdk_pixbuf_get_width (item->data.pixmap.pixbufs[0]);
       req->height = gdk_pixbuf_get_height (item->data.pixmap.pixbufs[0]);
     }
+
+  if (item->item_type == GREETER_ITEM_TYPE_SVG)
+    {
+      GdkPixbuf *svg;
+
+      svg = rsvg_pixbuf_from_file (item->data.pixmap.files[0], NULL);
+      req->width = gdk_pixbuf_get_width (svg);
+      req->height = gdk_pixbuf_get_height (svg);
+      g_object_unref (svg);
+    }
+ 
+  if (req->width > 0 && req->height > 0)
+    {
+      if (item->width_type == GREETER_ITEM_SIZE_SCALE && set_height > 0)
+        set_width = (req->width * set_height) / req->height;
+      else if (item->height_type == GREETER_ITEM_SIZE_SCALE && set_width > 0)
+        set_height = (req->height * set_width) / req->width;
+    }
   
   if (set_width > 0)
     req->width = set_width;
