@@ -1,3 +1,21 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <sys/types.h>
@@ -14,6 +32,8 @@
 
 #include "gdm.h"
 #include "gdmwm.h"
+#include "gdmcommon.h"
+#include "gdmsession.h"
 
 #include "greeter.h"
 #include "greeter_session.h"
@@ -25,57 +45,40 @@
 #define LAST_LANGUAGE "Last"
 #define SESSION_NAME "SessionName"
 
-static gboolean save_session = FALSE;
-static char *current_session = NULL;
-static gchar *default_session = NULL;
-static GSList *sessions = NULL;
-static GHashTable *sessnames = NULL;
-
-/* This is true if session dir doesn't exist or is whacked out
- * in some way or another */
-gboolean session_dir_whacked_out = FALSE;
+static gint save_session = GTK_RESPONSE_NO;
 static GtkWidget *session_dialog;
-
 static GSList *session_group = NULL;
 
-static gboolean 
-greeter_login_list_lookup (GSList *l, const gchar *data)
-{
-    GSList *list = l;
+extern GList *sessions;
+extern GHashTable *sessnames;
+extern gchar *default_session;
+extern char *current_session;
+extern gboolean session_dir_whacked_out;
 
-    if (!list || !data)
-	return(FALSE);
+static gboolean 
+greeter_login_list_lookup (GList *l, const gchar *data)
+{
+    GList *list = l;
+
+    if (list == NULL || data == NULL)
+	return FALSE;
+
+    /* FIXME: Hack, will support these builtin types later */
+    if (strcmp (data, GDM_SESSION_DEFAULT ".desktop") == 0 ||
+	strcmp (data, GDM_SESSION_CUSTOM ".desktop") == 0 ||
+	strcmp (data, GDM_SESSION_FAILSAFE ".desktop") == 0) {
+	    return TRUE;
+    }
 
     while (list) {
 
 	if (strcmp (list->data, data) == 0)
-	    return (TRUE);
+	    return TRUE;
 	
 	list = list->next;
     }
 
-    return (FALSE);
-}
-
-static const char *
-session_name (const char *name)
-{
-	const char *nm;
-
-	/* eek */
-	if (name == NULL)
-		return "(null)";
-
-	if (strcmp (name, GDM_SESSION_FAILSAFE_GNOME) == 0)
-		return _("Failsafe Gnome");
-	else if (strcmp (name, GDM_SESSION_FAILSAFE_XTERM) == 0)
-		return _("Failsafe xterm");
-
-	nm = g_hash_table_lookup (sessnames, name);
-	if (nm != NULL)
-		return nm;
-	else
-		return name;
+    return FALSE;
 }
 
 char *
@@ -84,20 +87,20 @@ greeter_session_lookup (const char *saved_session)
   gchar *session = NULL;
   
   /* Don't save session unless told otherwise */
-  save_session = FALSE;
+  save_session = GTK_RESPONSE_NO;
 
   /* Previously saved session not found in ~/.dmrc */
   if ( ! (saved_session != NULL &&
 	  strcmp ("(null)", saved_session) != 0 &&
 	  saved_session[0] != '\0')) {
-    /* If "Last" is chosen run Default,
+    /* If "Last" is chosen run default,
      * else run user's current selection */
     if (current_session == NULL || strcmp (current_session, LAST_SESSION) == 0)
       session = g_strdup (default_session);
     else
       session = g_strdup (current_session);
     
-    save_session = TRUE;
+    save_session = GTK_RESPONSE_YES;
     return session;
   }
 
@@ -117,9 +120,9 @@ greeter_session_lookup (const char *saved_session)
 				   "installed on this machine.\n"
 				   "Do you wish to make %s the default for "
 				   "future sessions?"),
-				 session_name (saved_session),
-				 session_name (default_session));	    
-	  save_session = greeter_query (msg, _("Make _Default"), _("Just _Log In"));
+				 gdm_session_name (saved_session),
+				 gdm_session_name (default_session));	    
+	  save_session = gdm_common_query (msg, FALSE /* markup */, _("Make _Default"), _("Just _Log In"), TRUE);
 	  g_free (msg);
 	}
     }
@@ -130,11 +133,10 @@ greeter_session_lookup (const char *saved_session)
       /* User's saved session is not the chosen one */
       if (strcmp (session, GDM_SESSION_FAILSAFE_GNOME) == 0 ||
 	  strcmp (session, GDM_SESSION_FAILSAFE_XTERM) == 0 ||
-	  /* bad hack, "Failsafe" is just a name in the session dir */
-	  strcmp (session, "Failsafe.desktop") == 0 ||
-	  strcmp (session, "Failsafe") == 0)
+	  g_ascii_strcasecmp (session, GDM_SESSION_FAILSAFE ".desktop") == 0 ||
+	  g_ascii_strcasecmp (session, GDM_SESSION_FAILSAFE) == 0)
 	{
-	  save_session = FALSE;
+	  save_session = GTK_RESPONSE_YES;
 	}
       else if (strcmp (saved_session, session) != 0)
 	{
@@ -147,12 +149,13 @@ greeter_session_lookup (const char *saved_session)
 				       "setting is %s.\nDo you wish "
 				       "to make %s the default for "
 				       "future sessions?"),
-				     session_name (session),
-				     session_name (saved_session),
-				     session_name (session));
-	      save_session = greeter_query (msg, _("Make _Default"), _("Just For _This Session"));
+				     gdm_session_name (session),
+				     gdm_session_name (saved_session),
+				     gdm_session_name (session));
+	      save_session = gdm_common_query (msg, FALSE /* markup */, _("Make _Default"), _("Just For _This Session"), TRUE);
 	    }
-	  else if (strcmp (session, "Xclients.desktop") != 0 &&
+	  else if (strcmp (session, default_session) != 0 &&
+		   strcmp (session, saved_session) != 0 &&
 		   strcmp (session, LAST_SESSION) != 0)
 	    {
 	      /* if !GdmShowLastSession then our saved session is
@@ -160,16 +163,19 @@ greeter_session_lookup (const char *saved_session)
 	       * and the relevant thing is the saved session
 	       * in .Xclients
 	       */
-	      msg = g_strdup_printf (_("You have chosen %s for this "
-				       "session.\nIf you wish to make %s "
-				       "the default for future sessions,\n"
-				       "run the 'switchdesk' utility\n"
-				       "(System->Desktop Switching Tool from "
-				       "the panel menu)."),
-				     session_name (session),
-				     session_name (session));
-	      save_session = FALSE;
-	      greeter_message (msg);
+	      if (access ("/usr/bin/switchdesk", F_OK) == 0)
+	        {
+	          msg = g_strdup_printf (_("You have chosen %s for this "
+				           "session.\nIf you wish to make %s "
+				           "the default for future sessions,\n"
+				           "run the 'switchdesk' utility\n"
+					   "(System->Desktop Switching Tool from "
+					   "the panel menu)."),
+					 gdm_session_name (session),
+					 gdm_session_name (session));
+		  gdm_common_message (msg);
+		}
+	      save_session = GTK_RESPONSE_NO;
 	    }
 	  g_free (msg);
 	}
@@ -178,7 +184,7 @@ greeter_session_lookup (const char *saved_session)
   return session;
 }
 
-gboolean
+gint
 greeter_save_session (void)
 {
   return save_session;
@@ -194,13 +200,12 @@ greeter_session_init (void)
   GtkWidget *cat_vbox = NULL;
   GtkWidget *radio;
   GtkWidget *dialog;
-  DIR *sessdir;
-  struct dirent *dent;
-  gboolean searching_for_default = FALSE;
+  GList *tmp;
   static GtkTooltips *tooltips = NULL;
   GtkRequisition req;
   char *s;
   int num = 1;
+  char *label;
 
   g_free (current_session);
   current_session = NULL;
@@ -251,8 +256,6 @@ greeter_session_init (void)
   vbox = gtk_vbox_new (FALSE, 6);
   /* we will pack this later depending on size */
 
-  sessnames = g_hash_table_new (g_str_hash, g_str_equal);
-
   if (GdmShowLastSession)
     {
       current_session = g_strdup (LAST_SESSION);
@@ -269,185 +272,43 @@ greeter_session_init (void)
       gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
       gtk_widget_show (radio);
     }
-    
-    /* Check that session dir is readable */
-    if (GdmSessionDir == NULL ||
-	access (GdmSessionDir, R_OK|X_OK))
+
+    gdm_session_list_init();
+
+    for (tmp = sessions; tmp != NULL; tmp = tmp->next)
       {
-	syslog (LOG_ERR, _("%s: Session directory %s not found!"), "gdm_login_session_init", ve_sure_string (GdmSessionDir));
-	session_dir_whacked_out = TRUE;
-	GdmShowXtermFailsafeSession = TRUE;
-      }
-
-    /* Read directory entries in session dir */
-    if (GdmSessionDir == NULL)
-	    sessdir = NULL;
-    else
-	    sessdir = opendir (GdmSessionDir);
-
-    if (sessdir != NULL)
-	    dent = readdir (sessdir);
-    else
-	    dent = NULL;
-
-    while (dent != NULL) {
-	    VeConfig *cfg;
-	    char *exec;
-	    char *name;
-	    char *comment;
-	    char *label;
-	    /* ignore everything bug the .desktop files */
-	    if (strstr (dent->d_name, ".desktop") == NULL) {
-		    dent = readdir (sessdir);
-		    continue;
-	    }
-
-	    s = g_strconcat (GdmSessionDir, "/", dent->d_name, NULL);
-	    cfg = ve_config_new (s);
-	    g_free (s);
-
-	    exec = ve_config_get_string (cfg, "Desktop Entry/Exec");
-	    name = ve_config_get_translated_string (cfg, "Desktop Entry/Name");
-	    comment = ve_config_get_translated_string (cfg, "Desktop Entry/Comment");
-
-	    ve_config_destroy (cfg);
-
-	    if (ve_string_empty (exec) || ve_string_empty (name)) {
-		    g_free (exec);
-		    g_free (name);
-		    g_free (comment);
-		    dent = readdir (sessdir);
-		    continue;
-	    }
-
-	    if (num < 10)
-		    label = g_strdup_printf ("_%d. %s", num, name);
-	    else
-		    label = g_strdup (name);
-	    num ++;
-
-	    radio = gtk_radio_button_new_with_mnemonic (session_group, label);
-	    g_free (label);
-	    g_object_set_data_full (G_OBJECT (radio),
-				    SESSION_NAME,
-				    g_strdup (dent->d_name),
-				    (GDestroyNotify) g_free);
-	    session_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
-	    gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
-	    gtk_widget_show (radio);
-
-	    if ( ! ve_string_empty (comment))
-		    gtk_tooltips_set_tip
-			    (tooltips, GTK_WIDGET (radio), comment, NULL);
-
-
-	    sessions = g_slist_append (sessions, g_strdup (dent->d_name));
-
-	    /* if we found the default session */
-	    if ( ! ve_string_empty (GdmDefaultSession) &&
-		 strcmp (dent->d_name, GdmDefaultSession) == 0) {
-		    g_free (default_session);
-		    default_session = g_strdup (dent->d_name);
-		    searching_for_default = FALSE;
-	    }
-
-	    /* if there is a session called Default */
-	    if (searching_for_default &&
-		g_ascii_strcasecmp (dent->d_name, "Default.desktop") == 0) {
-		    g_free (default_session);
-		    default_session = g_strdup (dent->d_name);
-	    }
-
-	    if (searching_for_default &&
-		g_ascii_strcasecmp (dent->d_name, "Gnome.desktop") == 0) {
-		    /* Just in case there is no Default session and
-		     * no default link, make Gnome the default */
-		    if (default_session == NULL)
-			    default_session = g_strdup (dent->d_name);
-	    }
-
-	    g_hash_table_insert (sessnames, g_strdup (dent->d_name), name);
-
-	    g_free (exec);
-	    g_free (comment);
-
-	    dent = readdir (sessdir);
-    }
-
-    if (sessdir != NULL)
-	    closedir (sessdir);
-
-    if (sessions == NULL)
-      {
-	syslog (LOG_WARNING, _("Yaikes, nothing found in the session directory."));
-	session_dir_whacked_out = TRUE;
-	GdmShowXtermFailsafeSession = TRUE;
-	
-	default_session = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
-      }
-
-    if (GdmShowGnomeFailsafeSession)
-      {
-	/* For translators:  This is the failsafe login when the user
-	 * can't login otherwise */
-	radio = gtk_radio_button_new_with_mnemonic (session_group,
-						    _("Failsafe _Gnome"));
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (radio),
-			      _("This is a failsafe session that will log you "
-				"into GNOME.  No startup scripts will be read "
-				"and it is only to be used when you can't log "
-				"in otherwise.  GNOME will use the 'Default' "
-				"session."),
-			      NULL);
-	g_object_set_data (G_OBJECT (radio),
-			   SESSION_NAME, GDM_SESSION_FAILSAFE_GNOME);
-	
+	GdmSession *session;
+	char *file;
+ 
+	file = (char *) tmp->data;
+	session = g_hash_table_lookup (sessnames, file);
+ 
+	if (num < 10 &&
+	   (strcmp (file, GDM_SESSION_FAILSAFE_GNOME) != 0) &&
+	   (strcmp (file, GDM_SESSION_FAILSAFE_XTERM) != 0))
+		label = g_strdup_printf ("_%d. %s", num, session->name);
+	else
+		label = g_strdup (session->name);
+	num ++;
+ 
+	radio = gtk_radio_button_new_with_mnemonic (session_group, label);
+	g_free (label);
+	g_object_set_data_full (G_OBJECT (radio), SESSION_NAME,
+		file, (GDestroyNotify) g_free);
 	session_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
-	sessions = g_slist_append (sessions,
-				   g_strdup (GDM_SESSION_FAILSAFE_GNOME));
-	
 	gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
 	gtk_widget_show (radio);
-      }
-
-    if (GdmShowXtermFailsafeSession)
-      {
-	/* For translators:  This is the failsafe login when the user
-	 * can't login otherwise */
-	radio = gtk_radio_button_new_with_mnemonic (session_group,
-						    _("Failsafe _Terminal"));
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (radio),
-			      _("This is a failsafe session that will log you "
-				"into a terminal.  No startup scripts will be read "
-				"and it is only to be used when you can't log "
-				"in otherwise.  To exit the terminal, "
-				"type 'exit'."),
-			      NULL);
-	g_object_set_data (G_OBJECT (radio),
-			   SESSION_NAME, GDM_SESSION_FAILSAFE_XTERM);
-
-	session_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
-	sessions = g_slist_append (sessions,
-				   g_strdup (GDM_SESSION_FAILSAFE_XTERM));
-	
-	gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 4);
-	gtk_widget_show (radio);
-      }
+ 
+	if (! ve_string_empty (session->comment))
+		gtk_tooltips_set_tip
+			(tooltips, GTK_WIDGET (radio), session->comment, NULL);
+     }
                     
-    if (default_session == NULL)
-      {
-	default_session = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
-	syslog (LOG_WARNING, _("No default session link found. Using Failsafe GNOME.\n"));
-      }
-    
-    if (current_session == NULL)
-            current_session = g_strdup (default_session);
+   gtk_widget_show_all (vbox);
+   gtk_widget_size_request (vbox, &req);
 
-    gtk_widget_show_all (vbox);
-    gtk_widget_size_request (vbox, &req);
-
-    /* if too large */
-    if (req.height > 0.7 * gdm_wm_screen.height) {
+   /* if too large */
+   if (req.height > 0.7 * gdm_wm_screen.height) {
 	    GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
 	    gtk_widget_set_size_request (sw,
 					 req.width, 
@@ -465,11 +326,11 @@ greeter_session_init (void)
 	    gtk_box_pack_start (GTK_BOX (hbox),
 				sw,
 				TRUE, TRUE, 0);
-    } else {
+   } else {
 	    gtk_box_pack_start (GTK_BOX (hbox),
 				vbox,
 				TRUE, TRUE, 0);
-    }
+   }
 }
 
 static void
